--[[##################################################################################

	PXControl by alex82
	Based on Ptokax Remote Administration by Hungarista

####################################################################################

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

###################################################################################]]
--_start = os.clock()
--_log_cmd = true

tData = {
	iLastPage = 1,		-- The tree-item number which is selected
	tDeadThreads = {},                -- The threads, which can be destroyed
	tSortedLists = {},                -- Sorted lists

	iChkMax = 63,
	iNumMax = 113,
	iTxtMax = 36,
	iPermMax = 55,
	
	iIP2CFilePart = 65000,
}

tStyle = {
	Chat = {
		Default = {
			Font = "Microsoft Sans Serif",
			Size = 8,
			Italic = 0,
			Bold = 0,
			Color = {0,0,100}
		},
		Time = {
			Color = {0,0,0}
		},
		Nick = {
			Bold = 1,
		},
		OpNick = {
			Bold = 1,
			Color = {200,0,0}
		},
		MyNick = {
			Bold = 1,
			Color = {0,128,0}
		},
		PM = {
			Color = {255,0,0}
		},
	},
}
--##################################################################################

_APPNAME = 'PXControl'
_APPVER = '1.02'

_path = debug.getinfo(1).source:match("^@?(.+[/\\]).-$")	-- Program path
if _path then
	_path = _path:gsub("\\","/")
else
	_path = "./"
end
_save_path = _path	-- Data saving path

package.path = _path.."lib/?.lua"
package.cpath = _path..'lib/?.so;'.._path..'lib/?.dll;'

sTag = "<PXControl V:".._APPVER:gsub("%s","")..",M:A,H:1/0/1,S:10>",
require 'wx'     -- GUI drawing
if _path == "./" then
	_path = wx.wxGetCwd():gsub("\\","/")
	if _path:sub(-1, -1) ~= "/" then
		_path = _path.."/"
	end
	_save_path = _path

	package.path = _path.."lib/?.lua"
	package.cpath = _path..'lib/?.so;'.._path..'lib/?.dll;'
end
require 'lfs'
require 'socket.core' -- Sockets
if wx.__WXMSW__ then require "process" 
else os.setlocale(wx.wxLocale():GetCanonicalName()) end

if wx.__WXMSW__ then
	local f = io.open(_path.."portable.txt")
	if f then
		f:close()
	else
		_save_path = wx.wxStandardPaths.Get():GetUserConfigDir():gsub("\\","/").."/PXControl/"
	end
else
	_save_path = wx.wxStandardPaths.Get():GetConfigDir():gsub("\\","/").."/pxcontrol/"
end
if not wx.wxDirExists(_save_path) then
	wx.wxMkdir(_save_path, 0755)
end

tCfg = {
	DefaultNick = "Admin",
	UDPDebug = false,
	ScriptErrors = true,
	UDPDebugPort = 422,
	AdvancedEditor = true,
	TrayIcon = true,
	MinToTray = true,
	DoubleClickDownload = true,
	Reconnect = true,
}

tHubs = {
	{Name = " ",Addr = "127.0.0.1"},
}

--##################################################################################

dofile(_path..'lua/Save.lua')
pcall(dofile, _save_path.."Config.tbl")
pcall(dofile, _save_path.."Hubs.tbl")

tData.TrayIcon = tCfg.TrayIcon

function Log(data)
	local f = io.open(_save_path.."log.txt","a")
	if f then
		f:write(os.date("[%Y-%m-%d %H:%M:%S] "), data, "\n")
		f:close()
	end
end

function LangSelect()
	local langs = {}
	for file in lfs.dir(_path.."language/") do
		if file:sub(-4, -1):lower() == ".tbl" then
			table.insert(langs, file:sub(1, -5))
		end
	end
	if #langs == 1 then
		tCfg.Language = langs[1]
		return
	end

	local dialog = wx.wxDialog(wx.NULL,-1,"Choose your language",wx.wxDefaultPosition,wx.wxSize(176,90),wx.wxCAPTION+wx.wxSYSTEM_MENU+wx.wxTAB_TRAVERSAL)
	local langsel = wx.wxChoice(dialog,-1,wx.wxPoint(5,5),wx.wxSize(160,-1))
	local ok = wx.wxButton(dialog,-1,"OK",wx.wxPoint(5,30),wx.wxSize(160,-1))

	local cur_lang = 1
	for i, lang in ipairs(langs) do
		if lang:lower() == "english" then
			cur_lang = i
		end
		langsel:Append(lang)
	end
	langsel:SetSelection(cur_lang-1)

	ok:Connect(wx.wxEVT_COMMAND_BUTTON_CLICKED,
		function(event)
			tCfg.Language = langs[langsel:GetCurrentSelection()+1]
			dialog:Close()
		end)
	dialog:Centre()
	dialog:ShowModal()
end

if tCfg.Language then
	pcall(dofile, _path..'language/'..tCfg.Language..'.tbl')
end
if not tCfg.Language or not tLang then
	LangSelect()
end
assert(tCfg.Language, "Language files not found")
dofile(_path..'language/'..tCfg.Language..'.tbl')
assert(tLang, "Invalid language file")

if wx.__WXMSW__ then
	os.setlocale(tLang.ProgramLocale or "C")
end
os.setlocale("C","numeric")

setmetatable(tLang,{__index = function(t,k)
	if k and not rawget(t,k) then
		local ret = "<"..k..">"
		rawset(t,k,ret)
		return ret
	end
end})

dofile(_path..'lua/Debug.lua')

mutex = ThreadNewMutex()

dofile(_path..'lua/Funcs.lua')
dofile(_path..'lua/Protocol.lua')
tFuncs.InitHubData()

dofile(_path..'lua/Dialogs.lua')
dofile(_path..'lua/UDPDebug.lua')
dofile(_path..'lua/Gui.lua')
InitGUI()

dofile(_path..'lua/GuiFunctions.lua')
dofile(_path..'lua/Translation.lua')
TranslateGUI()

collectgarbage("collect")

tFuncs.SetStatusText(tLang.Ready)
tCtrls["app"]:Centre()

if tCfg.Maximized then
	tCtrls["app"]:Maximize(true)
end
tCtrls["app"]:Show(true)
if tCfg.UDPDebug then
	tFuncs.UDPDebugOpen()
	tCtrls["MAINMENU_UDPDBG"]:Check(true)
end
if tCfg.ScriptErrors then
	tCtrls["MAINMENU_SCRIPTERRORS"]:Check(true)
end
--tFuncs.SetStatusText(os.clock()-_start.." sec.")	-- Program loading time
if tCfg.Connect and tCfg.HubNumber and tHubs[tCfg.HubNumber] then
	tCfg.Connect = nil
	tFuncs.Connect()
end
wx.wxGetApp():MainLoop()
