--[[##################################################################################

	PXControl by alex82
	Based on Ptokax Remote Administration by Hungarista

####################################################################################

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

###################################################################################]]

tCounters = {}
tNames = {}
tDebugArray = {}

local hook = function()
	local f = debug.getinfo(2,"f").func
	if not tNames[f] then
		if #tDebugArray > 200 then table.remove(tDebugArray,1) end
		table.insert(tDebugArray,f) -- Order the stack called by time - first call
		tNames[f] = debug.getinfo(2, "Snl")
	end
end
debug.sethook(hook,"c") -- Set the hook

function OnError(errorstr)
	local file = io.open(_save_path.."exceptioninfo_"..os.date("%m-%d_%H.%M.%S")..".txt","a+")
	if file then
		local getname = function(func)
			local n = tNames[func]
			if n.what == "C" then return n.name end
			local loc = string.format("[%s]:%s",n.short_src, n.linedefined)
			if n.namewhat ~= "" then return string.format("%s [%s] (%s)", loc, n.currentline,n.name)
			else return string.format("%s", loc) end
		end
		local tUptime = os.date("*t",os.clock())
		file:write("System informations: \n"..
		"               Time: "..os.date("%x %X").."\n"..
		"                 OS: "..wx.wxGetOsDescription().."\n"..
		"        Free memory: "..wx.wxGetFreeMemory():ToDouble().."\n"..
		"       Startup path: "..wx.wxGetCwd().."\n"..
		"         Resolution: "..wx.wxGetDisplaySize():GetWidth().."x"..wx.wxGetDisplaySize():GetHeight().."\n"..
		"             Uptime: "..string.format("%i hour(s) %i minute(s) %i second(s)",tUptime.hour,tUptime.min, tUptime.sec).."\n"..
		(sHubversion and 
		"     PtokaX Version: "..sHubversion.."\n" or "")..
		"    Program version: ".._APPVER.."\n"..
		"        LUA version: ".._VERSION.."\n"..
		"  wxWidgets version: "..wx.wxVERSION_STRING.."\n"..
		"      wxLUA version: "..wxlua.wxLUA_VERSION_STRING.."\n"..
		string.rep("=",60).."\nError message description: "..tostring(errorstr).."\n"..
		"----------------\nStack traceback:\n----------------\n")
		for i,arr in ipairs(tDebugArray) do
			file:write(tostring(getname(arr)).."\n")
		end
		file:write(string.rep("=",60).."\n")
		file:close()
	end
	if wx.wxMessageBox("An error occured in the application: "..errorstr.."\r\n\r\nTerminate the program?","ERROR",wx.wxICON_ERROR+wx.wxYES_NO+wx.wxYES_DEFAULT) == wx.wxYES then
		wx.wxExit(1)
	end
end
