--[[##################################################################################

	PXControl by alex82
	Based on Ptokax Remote Administration by Hungarista

####################################################################################

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

###################################################################################]]

tEditor = {}

function tEditor.HasBit(value, num)
    for n = 32, 0, -1 do
        local b = 2^n
        local num_b = num - b
        local value_b = value - b
        if num_b >= 0 then
            num = num_b
        else
            return true -- already tested bits in num
        end
        if value_b >= 0 then
            value = value_b
        end
        if (num_b >= 0) and (value_b < 0) then
            return false
        end
    end

    return true
end

function tEditor.SetStyle()
	dofile(_path..'lua/EditorStyle.lu')
end

editor = wxstc.wxStyledTextCtrl(tFuncs.GetCtrl("ID_SPLITTERWINDOW2"), wx.wxID_HIGHEST + 1,wx.wxDefaultPosition, wx.wxDefaultSize,0)
    editor:SetBufferedDraw(true)
	
	tEditor.SetStyle()
	
    editor:SetUseTabs(false)
    editor:SetTabWidth(4)
    editor:SetIndent(4)
    editor:SetIndentationGuides(true)

    editor:SetVisiblePolicy(wxstc.wxSTC_VISIBLE_SLOP, 3)
    --editor:SetXCaretPolicy(wxstc.wxSTC_CARET_SLOP, 10)
    --editor:SetYCaretPolicy(wxstc.wxSTC_CARET_SLOP, 3)

    editor:SetMarginWidth(0, editor:TextWidth(32, "9999_")) -- line # margin

    editor:SetMarginWidth(1, 16) -- marker margin
    editor:SetMarginType(1, wxstc.wxSTC_MARGIN_SYMBOL)
    editor:SetMarginSensitive(1, true)

    editor:SetMarginWidth(2, 16) -- fold margin
    editor:SetMarginType(2, wxstc.wxSTC_MARGIN_SYMBOL)
    editor:SetMarginMask(2, wxstc.wxSTC_MASK_FOLDERS)
    editor:SetMarginSensitive(2, true)

    editor:SetFoldFlags(wxstc.wxSTC_FOLDFLAG_LINEBEFORE_CONTRACTED +
                        wxstc.wxSTC_FOLDFLAG_LINEAFTER_CONTRACTED)

    editor:SetProperty("fold", "1")
    editor:SetProperty("fold.compact", "1")
    editor:SetProperty("fold.comment", "1")

    local grey = wx.wxColour(128, 128, 128)
    editor:MarkerDefine(wxstc.wxSTC_MARKNUM_FOLDEROPEN,    wxstc.wxSTC_MARK_BOXMINUS, wx.wxWHITE, grey)
    editor:MarkerDefine(wxstc.wxSTC_MARKNUM_FOLDER,        wxstc.wxSTC_MARK_BOXPLUS,  wx.wxWHITE, grey)
    editor:MarkerDefine(wxstc.wxSTC_MARKNUM_FOLDERSUB,     wxstc.wxSTC_MARK_VLINE,    wx.wxWHITE, grey)
    editor:MarkerDefine(wxstc.wxSTC_MARKNUM_FOLDERTAIL,    wxstc.wxSTC_MARK_LCORNER,  wx.wxWHITE, grey)
    editor:MarkerDefine(wxstc.wxSTC_MARKNUM_FOLDEREND,     wxstc.wxSTC_MARK_BOXPLUSCONNECTED,  wx.wxWHITE, grey)
    editor:MarkerDefine(wxstc.wxSTC_MARKNUM_FOLDEROPENMID, wxstc.wxSTC_MARK_BOXMINUSCONNECTED, wx.wxWHITE, grey)
    editor:MarkerDefine(wxstc.wxSTC_MARKNUM_FOLDERMIDTAIL, wxstc.wxSTC_MARK_TCORNER,  wx.wxWHITE, grey)
    grey:delete()

    editor:Connect(wxstc.wxEVT_STC_MARGINCLICK,
            function (event)
                local line = editor:LineFromPosition(event:GetPosition())
                local margin = event:GetMargin()
                if margin == 2 then
                    if wx.wxGetKeyState(wx.WXK_SHIFT) and wx.wxGetKeyState(wx.WXK_CONTROL) then
                        FoldSome()
                    else
                        local level = editor:GetFoldLevel(line)
                        if tEditor.HasBit(level, wxstc.wxSTC_FOLDLEVELHEADERFLAG) then
                            editor:ToggleFold(line)
                        end
                    end
                end
            end)

    editor:Connect(wxstc.wxEVT_STC_CHARADDED,
            function (event)
                -- auto-indent
                local ch = event:GetKey()
                if (ch == char_CR) or (ch == char_LF) then
                    local pos = editor:GetCurrentPos()
                    local line = editor:LineFromPosition(pos)

                    if (line > 0) and (editor:LineLength(line) == 0) then
                        local indent = editor:GetLineIndentation(line - 1)
                        if indent > 0 then
                            editor:SetLineIndentation(line, indent)
                            editor:GotoPos(pos + indent)
                        end
                    end
                end
            end)

    editor:Connect(wxstc.wxEVT_STC_USERLISTSELECTION,
            function (event)
                local pos = editor:GetCurrentPos()
                local start_pos = editor:WordStartPosition(pos, true)
                editor:SetSelection(start_pos, pos)
                editor:ReplaceSelection(event:GetText())
            end)

--[[    editor:Connect(wxstc.wxEVT_STC_SAVEPOINTREACHED,
            function (event)
                SetDocumentModified(editor:GetId(), false)
            end)

    editor:Connect(wxstc.wxEVT_STC_SAVEPOINTLEFT,
            function (event)
                SetDocumentModified(editor:GetId(), true)
            end)

    editor:Connect(wx.wxEVT_SET_FOCUS,
            function (event)
                event:Skip()
                if in_evt_focus or exitingProgram then return end
                in_evt_focus = true
                IsFileAlteredOnDisk(editor)
                in_evt_focus = false
            end)]]
			
	editor:SetLexer(wxstc.wxSTC_LEX_LUA)
	

	editor:SetKeyWords(0,
		[[and break do else elseif end false for function if
		in local nil not or repeat return then true until while]])
	editor:SetKeyWords(1,
		[[_VERSION assert collectgarbage dofile error gcinfo loadfile loadstring print tonumber tostring type unpack
		_ALERT _ERRORMESSAGE _INPUT _PROMPT _OUTPUT _STDERR _STDIN _STDOUT call dostring foreach foreachi getn globals
		newtype rawget rawset require sort tinsert tremove _G getfenv getmetatable ipairs loadlib next pairs pcall
		rawegal rawget rawset require setfenv setmetatable xpcall string table math coroutine io os debug]])
	editor:SetKeyWords(2,
		[[abs acos asin atan atan2 ceil cos deg exp floor format frexp gsub ldexp log log10 max min mod rad random
		randomseed sin sqrt strbyte strchar strfind strlen strlower strrep strsub strupper tan string.byte string.char
		string.dump string.find string.len string.lower string.rep string.sub string.upper string.format string.gfind
		string.gsub table.concat table.foreach table.foreachi table.getn table.sort table.insert table.remove table.setn
		math.abs math.acos math.asin math.atan math.atan2 math.ceil math.cos math.deg math.exp math.floor math.frexp
		math.ldexp math.log math.log10 math.max math.min math.mod math.pi math.rad math.random math.randomseed math.sin
		math.sqrt math.tan]])
	editor:SetKeyWords(3,
		[[os io coroutine debug package os.clock os.date os.difftime os.execute os.exit os.getenv os.remove os.rename
		os.setlocale os.time os.tmpname io.close io.flush io.input io.lines io.open io.output io.popen io.read io.stderr
		io.stdin io.stdout io.tmpfile io.type io.write coroutine.create coroutine.resume coroutine.running coroutine.status
		coroutine.wrap coroutine.yield debug.debug debug.getfenv debug.gethook debug.getinfo debug.getlocal
		debug.getmetatable debug.getregistry debug.getupvalue debug.setfenv debug.sethook debug.setlocal debug.setmetatable
		debug.setupvalue debug.traceback package.cpath package.loaded package.loaders package.loadlib package.path
		package.preload package.seeall]])
	editor:SetKeyWords(4,
		[[Core.Version Core.Restart Core.Shutdown Core.ResumeAccepts Core.SuspendAccepts Core.SuspendAccepts Core.RegBot
		Core.UnregBot Core.GetBots Core.GetActualUsersPeak Core.GetMaxUsersPeak Core.GetCurrentSharedSize Core.GetHubIP
		Core.GetHubSecAlias Core.GetPtokaXPath Core.GetUsersCount Core.GetUpTime Core.GetOnlineNonOps Core.GetOnlineNonOps
		Core.GetOnlineOps Core.GetOnlineOps Core.GetOnlineRegs Core.GetOnlineRegs Core.GetOnlineUsers Core.GetOnlineUsers
		Core.GetOnlineUsers Core.GetOnlineUsers Core.GetUser Core.GetUser Core.GetUserAllData Core.GetUserData
		Core.GetUserValue Core.GetUsers Core.GetUsers Core.Disconnect Core.Disconnect Core.Kick Core.Redirect
		Core.DefloodWarn Core.SendToAll Core.SendToNick Core.SendToOpChat Core.SendToOps Core.SendToProfile Core.SendToUser
		Core.SendPmToAll Core.SendPmToNick Core.SendPmToOps Core.SendPmToProfile Core.SendPmToUser SetMan.Save SetMan.GetMOTD
		SetMan.SetMOTD SetMan.GetBool SetMan.SetBool SetMan.GetNumber SetMan.SetNumber SetMan.GetString SetMan.SetString
		SetMan.GetMinShare SetMan.SetMinShare SetMan.SetMinShare SetMan.GetMaxShare SetMan.SetMaxShare SetMan.SetMaxShare
		SetMan.SetHubSlotRatio SetMan.GetOpChat SetMan.SetOpChat SetMan.GetHubBot SetMan.SetHubBot RegMan.Save
		RegMan.GetRegsByProfile RegMan.GetNonOps RegMan.GetOps RegMan.GetReg RegMan.GetRegs RegMan.AddReg RegMan.DelReg
		RegMan.ChangeReg RegMan.ClrRegBadPass BanMan.Save BanMan.GetBans BanMan.GetTempBans BanMan.GetPermBans BanMan.GetBan
		BanMan.GetPermBan BanMan.GetTempBan BanMan.GetRangeBans BanMan.GetTempRangeBans BanMan.GetPermRangeBans
		BanMan.GetRangeBan BanMan.GetRangePermBan BanMan.GetRangeTempBan BanMan.Unban BanMan.UnbanPerm BanMan.UnbanTemp
		BanMan.UnbanAll BanMan.UnbanPermAll BanMan.UnbanTempAll BanMan.RangeUnban BanMan.RangeUnbanPerm
		BanMan.RangeUnbanTemp BanMan.ClearBans BanMan.ClearPermBans BanMan.ClearTempBans BanMan.ClearRangeBans
		BanMan.ClearRangePermBans BanMan.ClearRangeTempBans BanMan.Ban BanMan.BanIP BanMan.BanNick BanMan.TempBan
		BanMan.TempBanIP BanMan.TempBanNick BanMan.RangeBan BanMan.RangeTempBan ProfMan.AddProfile ProfMan.RemoveProfile
		ProfMan.MoveDown ProfMan.MoveUp ProfMan.GetProfile ProfMan.GetProfiles ProfMan.GetProfilePermission
		ProfMan.GetProfilePermissions ProfMan.SetProfileName ProfMan.SetProfilePermission TmrMan.AddTimer TmrMan.AddTimer
		TmrMan.RemoveTimer UDPDbg.Reg UDPDbg.Unreg UDPDbg.Send ScriptMan.GetScript ScriptMan.GetScripts ScriptMan.MoveUp
		ScriptMan.MoveDown ScriptMan.StartScript ScriptMan.RestartScript ScriptMan.StopScript ScriptMan.Restart
		ScriptMan.Refresh IP2Country.GetCountryCode IP2Country.GetCountryName IP2Country.GetCountryName]])
    editor:Colourise(0, -1)
	editor:Connect(wx.wxID_ANY,wx.wxEVT_CONTEXT_MENU,function(event)
		local editable = editor:GetSelectionEnd()-editor:GetSelectionStart() > 0
		local menu = wx.wxMenu()
		menu:Append(10,tLang.Undo):Enable(editor:CanUndo())
		menu:Append(11,tLang.Redo):Enable(editor:CanRedo())
		menu:AppendSeparator()
		menu:Append(12,tLang.Cut):Enable(editable)
		menu:Append(13,tLang.Copy):Enable(editable)
		menu:Append(14,tLang.Paste):Enable(editor:CanPaste())
		menu:Append(15,tLang.Delete):Enable(editable)
		menu:AppendSeparator()
		menu:Append(16,tLang.SelectAll)
		editor:PopupMenu(menu)
	end)

