--[[##################################################################################

	PXControl by alex82
	Based on Ptokax Remote Administration by Hungarista

####################################################################################

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

###################################################################################]]

tValidators = {
	-- TextValidators
	Nick = wx.wxTextValidator(wx.wxFILTER_EXCLUDE_CHAR_LIST),	-- For NMDC nickname
	NoPipe = wx.wxTextValidator(wx.wxFILTER_EXCLUDE_CHAR_LIST),	-- NoPipe
	NoPipeDollar = wx.wxTextValidator(wx.wxFILTER_EXCLUDE_CHAR_LIST),	-- NoPipeDollar
	Descr = wx.wxTextValidator(wx.wxFILTER_EXCLUDE_CHAR_LIST),
	Ports = wx.wxTextValidator(wx.wxFILTER_INCLUDE_CHAR_LIST),
	Numbers = wx.wxTextValidator(wx.wxFILTER_INCLUDE_CHAR_LIST),
--	Address = wx.wxTextValidator(wx.wxFILTER_INCLUDE_CHAR_LIST),
--	Email = wx.wxTextValidator(wx.wxFILTER_INCLUDE_CHAR_LIST),
	IPv4 = wx.wxTextValidator(wx.wxFILTER_INCLUDE_CHAR_LIST),
	IPv6 = wx.wxTextValidator(wx.wxFILTER_INCLUDE_CHAR_LIST),
}
tValidators.Nick:SetExcludes({" ","|","$"})
tValidators.Password = tValidators.Nick
tValidators.NoPipe:SetExcludes({"|"})
tValidators.NoPipeDollar:SetExcludes({"|","$"})
tValidators.Descr:SetExcludes({"|","$","<",">"})
local val = {"0","1","2","3","4","5","6","7","8","9"}
tValidators.Numbers:SetIncludes(val)
table.insert(val, ";")
tValidators.Ports:SetIncludes(val)
table.remove(val)
table.insert(val, ".")
tValidators.IPv4:SetIncludes(val)
--[[table.remove(val)
table.insert(val, "-")
for i = 65, 90 do
	table.insert(val, string.char(i))
end
for i = 97, 122 do
	table.insert(val, string.char(i))
end
tValidators.Address:SetIncludes(val)
table.insert(val, "@")
tValidators.Email:SetIncludes(val)]]
val = nil
tValidators.IPv6:SetIncludes({"0","1","2","3","4","5","6","7","8","9",":","A","B","C","D","E","F","a","b","c","d","e","f"})

tCtrls = {
	["txt"] = {},	-- wxTextCtrl's
	["chk"] = {},	-- wxCheckBox's
	["num"] = {},	-- wxSpinCtrl's (num up/down)
	-- Others
	imglist = wx.wxImageList(16,16),  -- Imagelist
}

setmetatable(tCtrls,{__index = function(t,k)
	if k and not rawget(t,k) and k:sub(1,4) == "wxID" or k:sub(1,3) == "ID_" then
		local NewId = tCtrls.xmlResource.GetXRCID(k)
		if not NewId then NewId = wx.wxNewId() end -- Not used now
		rawset(t,k,NewId)
		return NewId
	end
end})

for i,v in ipairs({"dc.ico","set.ico","lua.ico","usr.ico","ban.ico","txt.ico","udp.ico"}) do
	tCtrls["imglist"]:Add(wx.wxIcon(_path.."resources/"..v,wx.wxBITMAP_TYPE_ICO))
end

if not wx.wxDirExists(_save_path.."scripts") then wx.wxMkdir(_save_path.."scripts") end
if not wx.wxDirExists(_save_path.."texts") then wx.wxMkdir(_save_path.."texts") end

function InitGUI()
	-- try to load the resource and ask for path to it if not found
	tCtrls["xmlResource"] = wx.wxXmlResource()
	tCtrls["xmlResource"]:InitAllHandlers()
	if not tCtrls["xmlResource"]:Load(_path.."resources/Main.xrc") then
		wx.wxMessageBox(tLang.ErrorLoadingResources,_APPNAME,wx.wxICON_EXCLAMATION)
		return -- quit
	end

	tCtrls["xmlDialogs"] = wx.wxXmlResource()
	tCtrls["xmlDialogs"]:InitAllHandlers()
	if not tCtrls["xmlDialogs"]:Load(_path.."resources/Dialogs.xrc") then
		wx.wxMessageBox(tLang.ErrorLoadingResources,_APPNAME,wx.wxICON_EXCLAMATION)
		return -- quit
	end

	tCtrls["app"] = wx.wxFrame()
	if not tCtrls["xmlResource"]:LoadFrame(tCtrls["app"],wx.NULL,"ID_DLG_MAIN") then
		wx.wxMessageBox(tLang.ErrorLoadingResources,_APPNAME,wx.wxICON_EXCLAMATION)
		return -- quit program
	end
	tCtrls["app"]:SetMinSize(wx.wxSize(655,610))
	if tCfg.Width and tCfg.Height then tCtrls["app"]:SetSize(wx.wxSize(tCfg.Width,tCfg.Height)) end
	-- Seems everything is OK, find the controls...
	tCtrls["xmlResource"]:InitAllHandlers()
	tCtrls["statusbar"] = tFuncs.GetCtrl("ID_STATUSBAR")
	tCtrls["mainchat"] = tFuncs.GetCtrl("txt_chat")
	tCtrls["mainchat_msg"] = tFuncs.GetCtrl("txt_chat_msg")
	tCtrls["txt_status"] = tFuncs.GetCtrl("txt_status")
	
	tCtrls["ChatStyle"] = tCtrls["mainchat"]:GetDefaultStyle()
	tCtrls.ChatFonts = {}
	tCtrls.ChatColors = {}
	for v in pairs(tStyle.Chat) do
		local bold = wx.wxFONTWEIGHT_BOLD
		if tStyle.Chat[v].Bold then
			if tStyle.Chat[v].Bold == 0 then
				bold = wx.wxFONTWEIGHT_NORMAL
			end
		else
			if tStyle.Chat.Default.Bold == 0 then
				bold = wx.wxFONTWEIGHT_NORMAL
			end

		end
		local ita = wx.wxFONTSTYLE_ITALIC
		if tStyle.Chat[v].Italic then
			if tStyle.Chat[v].Italic == 0 then
				ita = wx.wxFONTSTYLE_NORMAL
			end
		else
			if tStyle.Chat.Default.Italic == 0 then
				ita = wx.wxFONTSTYLE_NORMAL
			end
		end
		tCtrls.ChatFonts[v] = wx.wxFont(tStyle.Chat[v].Size or tStyle.Chat.Default.Size,wx.wxFONTFAMILY_DEFAULT,
			ita,bold,false,tStyle.Chat[v].Font or tStyle.Chat.Default.Font)
		tCtrls.ChatColors[v] = wx.wxColour(tStyle.Chat[v].Color and tStyle.Chat[v].Color[1] or tStyle.Chat.Default.Color[1],
			tStyle.Chat[v].Color and tStyle.Chat[v].Color[2] or tStyle.Chat.Default.Color[2],
			tStyle.Chat[v].Color and tStyle.Chat[v].Color[3] or tStyle.Chat.Default.Color[3])
	end
	tCtrls["mainchat"]:SetDefaultStyle(tCtrls["ChatStyle"])
	
	tCtrls["main_icon"] = wx.wxIcon(_path.."resources/spider.ico",wx.wxBITMAP_TYPE_ICO)
	
	if tData.TrayIcon then
		tCtrls["taskbar"] = wx.wxTaskBarIcon()	
		tCtrls["icon_r"] = wx.wxIcon(_path.."resources/r.ico",wx.wxBITMAP_TYPE_ICO)
		tCtrls["icon_y"] = wx.wxIcon(_path.."resources/y.ico",wx.wxBITMAP_TYPE_ICO)
		tCtrls["icon_g"] = wx.wxIcon(_path.."resources/g.ico",wx.wxBITMAP_TYPE_ICO)
		tCtrls["taskbar"]:SetIcon(tCtrls["icon_r"],_APPNAME.." ".._APPVER)
	end

	tCtrls["app"]:SetIcon(tCtrls["main_icon"])
	tCtrls["app"]:SetTitle(_APPNAME.." ".._APPVER)
	tCtrls["tab"] = tFuncs.GetCtrl("ID_NOTEBOOK")
	tCtrls["tab"]:SetImageList(tCtrls["imglist"],wx.wxIMAGE_LIST_SMALL)
	for i = 0,5 do
		tCtrls["tab"]:SetPageImage(i,i)
	end
	tCtrls["tab"]:RemovePage(6) -- Hidden pages don't work. Any idea why?
	tCtrls["hub_select"] = tFuncs.GetCtrl("wxID_HUB")
	for i,v in ipairs(tHubs) do
		tCtrls["hub_select"]:Append(v.Name)
	end
	if tCfg.HubNumber and tHubs[tCfg.HubNumber] then	
		tCtrls["hub_select"]:SetSelection(tCfg.HubNumber-1)
	else
		tCtrls["hub_select"]:SetSelection(0)
	end
	local mainmenu = wx.wxMenuBar()
	local menu = wx.wxMenu()
	menu:Append(tCtrls["ID_MAINMENU_SETTINGS"], tLang.Settings, tLang.OpenSettingsWindow)
	menu:AppendSeparator()
	menu:Append(tCtrls["ID_MAINMENU_EXIT"], tLang.Exit, tLang.ProgramExit)
	mainmenu:Append(menu, tLang.File)
	menu = wx.wxMenu()
	menu:Append(tCtrls["ID_MAINMENU_RELOADSTYLE"], tLang.ReloadStyles, tLang.ReloadEditorStyles)
	tCtrls["MAINMENU_UDPDBG"] = wx.wxMenuItem(menu,tCtrls["ID_MAINMENU_UDPDBG"], tLang.UDPDebugger, tLang.OpenUDPDebugger,wx.wxITEM_CHECK)
	menu:Append(tCtrls["MAINMENU_UDPDBG"])
	tCtrls["MAINMENU_SCRIPTERRORS"] = wx.wxMenuItem(menu,tCtrls["ID_MAINMENU_SCRIPTERRORS"], tLang.ScriptsErrors, tLang.ShowScriptErrors,wx.wxITEM_CHECK)
	menu:Append(tCtrls["MAINMENU_SCRIPTERRORS"])
	mainmenu:Append(menu, tLang.View)
	tCtrls["app"]:SetMenuBar(mainmenu)
	menu = wx.wxMenu()
	menu:Append(tCtrls["ID_MAINMENU_HOMEPAGE"], tLang.HomePage, tLang.VisitHomePage)
	menu:Append(tCtrls["ID_MAINMENU_ABOUT"], tLang.About, tLang.ShowAboutInfo)
	mainmenu:Append(menu, tLang.Help)
	tCtrls["app"]:SetMenuBar(mainmenu)
	
	for i=0,tData.iNumMax do tCtrls["num"]["num_"..i] = tFuncs.GetCtrl("num_"..i) end
	for i=0,tData.iTxtMax do tCtrls["txt"]["txt_"..i] = tFuncs.GetCtrl("txt_"..i) end
	tCtrls["txt"]["txt_motd"] = tFuncs.GetCtrl("txt_motd")
	for i=0,tData.iChkMax do tCtrls["chk"]["chk_"..i] = tFuncs.GetCtrl("chk_"..i) end
	tCtrls["settings_tree"] = tFuncs.GetCtrl("ID_TRC_SETTINGS")
	local root = tCtrls["settings_tree"]:AddRoot("", -1,-1)
	local tTree = {
	tCtrls["settings_tree"]:AppendItem(root,"General settings"),
	tCtrls["settings_tree"]:AppendItem(root,"Advanced"),
	tCtrls["settings_tree"]:AppendItem(root,"Rules"),
	tCtrls["settings_tree"]:AppendItem(root,"Deflood"),
	}
	table.insert(tTree,tCtrls["settings_tree"]:AppendItem(tTree[1],"MOTD"))
	table.insert(tTree,tCtrls["settings_tree"]:AppendItem(tTree[1],"Default bots"))
	table.insert(tTree,tCtrls["settings_tree"]:AppendItem(tTree[1],"More general"))
	table.insert(tTree,tCtrls["settings_tree"]:AppendItem(tTree[1],"Ban handling"))
	table.insert(tTree,tCtrls["settings_tree"]:AppendItem(tTree[2],"MyINFO processing"))
	table.insert(tTree,tCtrls["settings_tree"]:AppendItem(tTree[3],"More rules"))
	table.insert(tTree,tCtrls["settings_tree"]:AppendItem(tTree[4],"More deflood"))
	table.insert(tTree,tCtrls["settings_tree"]:AppendItem(tTree[4],"More more deflood"))
	-- Expand the trees
	tCtrls["settings_tree"]:Expand(tTree[1])
	tCtrls["settings_tree"]:Expand(tTree[2])
	tCtrls["settings_tree"]:Expand(tTree[3])
	tCtrls["settings_tree"]:Expand(tTree[4])
	tCtrls["settings_tree"]:Connect(wx.wxEVT_COMMAND_TREE_SEL_CHANGED,
	function(event)
		for i,v in ipairs(tTree) do
			if tCtrls["settings_tree"]:IsSelected(v) then
				tFuncs.GetCtrl("ID_PANEL"..tData.iLastPage):Show(false)
				tFuncs.GetCtrl("ID_PANEL"..i):Show(true)
				tData.iLastPage = i
				tFuncs.GetCtrl("ID_PNL_SETTINGS"):Layout()
				collectgarbage("collect")
			end
		end
	end)
	tCtrls["settings_tree"]:SelectItem(tTree[1])
	tCtrls["lst_scripts"] = tFuncs.GetCtrl("wxID_LUA_LIST")
	tCtrls["txt_scripterror"] = tFuncs.GetCtrl("txt_scripterror")
	if tCfg.AdvancedEditor then
		dofile(_path..'lua/Editor.lua')
	else
		editor = wx.wxTextCtrl(tFuncs.GetCtrl("ID_SPLITTERWINDOW2"), wx.wxID_HIGHEST + 1, "",wx.wxDefaultPosition, wx.wxDefaultSize,wx.wxTE_MULTILINE)
	end
	tFuncs.GetCtrl("ID_SPLITTERWINDOW2"):SplitHorizontally(editor,tCtrls["txt_scripterror"],tCfg.EditorSplit or 400)
	tFuncs.GetCtrl("ID_SPLITTERWINDOW2"):SetSashGravity(1)
	tCtrls["lst_regs"] = tFuncs.GetCtrl("wxID_REGLIST")
	tCtrls["lst_regs"]:InsertColumn(0,tLang.User,wx.wxLIST_FORMAT_LEFT,tCfg.RegCol_1 or 175)
	tCtrls["lst_regs"]:InsertColumn(1,tLang.Password,wx.wxLIST_FORMAT_LEFT,tCfg.RegCol_2 or 100)
	tCtrls["lst_regs"]:InsertColumn(2,tLang.Profile,wx.wxLIST_FORMAT_LEFT,tCfg.RegCol_3 or 100)
	tCtrls["lst_permissions"] = tFuncs.GetCtrl("wxID_PERMISSION_LIST")
	tCtrls["lst_profiles"] = tFuncs.GetCtrl("wxID_PROFLIST")
	tFuncs.InitTranslatableControls()
	tCtrls["lst_bans"] = tFuncs.GetCtrl("wxID_BANLIST")
	tCtrls["lst_bans"]:InsertColumn(0,tLang.IP,        wx.wxLIST_FORMAT_LEFT, tCfg.BanCol_1 or 120)
	tCtrls["lst_bans"]:InsertColumn(1,tLang.Nick,wx.wxLIST_FORMAT_LEFT, tCfg.BanCol_2 or 200)
	tCtrls["lst_bans"]:InsertColumn(2,tLang.BannedBy, wx.wxLIST_FORMAT_LEFT, tCfg.BanCol_3 or 130)
	tCtrls["lst_bans"]:InsertColumn(3,tLang.Reason,    wx.wxLIST_FORMAT_LEFT, tCfg.BanCol_4 or 230)
	tCtrls["lst_bans"]:InsertColumn(4,tLang.Expire,    wx.wxLIST_FORMAT_LEFT, tCfg.BanCol_5 or 130)
	tCtrls["lst_rangebans"] = tFuncs.GetCtrl("wxID_RANGEBANLIST")
	tCtrls["lst_rangebans"]:InsertColumn(0,tLang.Range,     wx.wxLIST_FORMAT_LEFT, tCfg.RangeBanCol_1 or 200)
	tCtrls["lst_rangebans"]:InsertColumn(1,tLang.BannedBy, wx.wxLIST_FORMAT_LEFT, tCfg.RangeBanCol_2 or 130)
	tCtrls["lst_rangebans"]:InsertColumn(2,tLang.Reason,    wx.wxLIST_FORMAT_LEFT, tCfg.RangeBanCol_3 or 300)
	tCtrls["lst_rangebans"]:InsertColumn(3,tLang.Expire,    wx.wxLIST_FORMAT_LEFT, tCfg.RangeBanCol_4 or 130)
	tCtrls["lst_texts"] = tFuncs.GetCtrl("wxID_TEXTS")
	
	tCtrls["txt"]["txt_1"]:SetValidator(tValidators.Nick)
--	tCtrls["txt"]["txt_2"]:SetValidator(tValidators.Address)
	tCtrls["txt"]["txt_3"]:SetValidator(tValidators.Ports)
	tCtrls["txt"]["txt_4"]:SetValidator(tValidators.Numbers)
	tCtrls["txt"]["txt_21"]:SetValidator(tValidators.Nick)
	tCtrls["txt"]["txt_22"]:SetValidator(tValidators.Descr)
	tCtrls["txt"]["txt_24"]:SetValidator(tValidators.Nick)
	tCtrls["txt"]["txt_25"]:SetValidator(tValidators.Descr)
--[[	tCtrls["txt"]["txt_23"]:SetValidator(tValidators.Email)
	tCtrls["txt"]["txt_26"]:SetValidator(tValidators.Email)
	tCtrls["txt"]["txt_30"]:SetValidator(tValidators.Email)]]
	tCtrls["txt"]["txt_35"]:SetValidator(tValidators.IPv4)
	tCtrls["txt"]["txt_36"]:SetValidator(tValidators.IPv6)
	
	tFuncs.CleanUpSettings()
	tFuncs.CleanUpProfiles()
	tFuncs.EnableControls(false)
end

