--[[##################################################################################

	PXControl by alex82
	Based on Ptokax Remote Administration by Hungarista

####################################################################################

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

###################################################################################]]

local tCommands, Entered
local tProtocolCmd = {["$SET"] = true, ["$GET"] = true, ["$ERR"] = true, ["$CONF"] = true}

tFuncs.InitHubData = function()
	tHubData = {
		tSettings = {       -- Settings. With this, just the modified settings will be sent.
			["chk"] = {},
			["txt"] = {},
			["num"] = {},
		},
		sScriptName = "",
		tOpList = {},
		tFiles = {},                      -- Files, which are currenty downloaded
		tBans = {},                       -- Bans (temp and perm too)
		tRangeBans = {},                  -- Rangebans (temp and perm too)
		tProfiles = {},                   -- Profiles
		tRegs = {},                       -- Registered users
		tTexts = {},
		tMax = {chk = 0, num = 0, txt = 0, perm = 0},
	}
end

tFuncs.Connect = function()
	local hubnumber = tCtrls["hub_select"]:GetSelection()+1
	if tHubs[hubnumber] then
		tData.CurrentHub = {
			Name = tHubs[hubnumber].Name,
			Nick = tHubs[hubnumber].Nick or tCfg.DefaultNick,
			Pass = tHubs[hubnumber].Pass or "",
			Desc = tHubs[hubnumber].Desc or tCfg.DefaultDesc or "",
			Tag = tHubs[hubnumber].Tag or sTag,
			Email = tHubs[hubnumber].Email or tCfg.DefaultEmail or "",
			Share = tHubs[hubnumber].Share or tCfg.DefaultShare or 0,
		}
		
		tData.CurrentHub.Address = tHubs[hubnumber].Addr
		tData.CurrentHub.Port = tHubs[hubnumber].Addr:match(":(%d+)$")
		if tData.CurrentHub.Port then
			tData.CurrentHub.Address = tHubs[hubnumber].Addr:match("(.+):")
		else
			tData.CurrentHub.Port = 411
		end

		tFuncs.GetCtrl("wxID_CMD_LOGIN"):SetLabel(tLang.Disconnect)
		tFuncs.GetCtrl("wxID_HUB"):Enable(false)
		tFuncs.GetCtrl("wxID_CMD_ADDHUB"):Enable(false)
		tFuncs.GetCtrl("wxID_CMD_MODHUB"):Enable(false)
		tFuncs.GetCtrl("wxID_CMD_DELHUB"):Enable(false)
		
		tData.bConnected = true

		tData.TCPThread = ThreadCreate(tFuncs.TCPListen,{})
	end
end

tFuncs.Send = function(msg)
	if tData.Client then
		local cmd = msg:match("^($%w+)")
		if cmd and tProtocolCmd[cmd] then
			tHubData.TryUseProtocol = true
		end
		msg = msg:gsub("|","[endpipe]")
		if cmd and _log_cmd then Log("<< "..msg) end
		tData.Client:settimeout(10)
		tData.Client:send(msg.."|")
		tData.Client:settimeout(nil)
	end
end

tFuncs.GetStats = function()
	if tHubData.tSettings["txt"][29]:sub(1,1) then tFuncs.Send("<"..tData.CurrentHub.Nick.."> "..tHubData.tSettings["txt"][29]:sub(1,1).."stats") end
end

tFuncs.TCPListen = function()
	local wait
	tData.bReconnect = true
	while tData.bReconnect do
		local res
		while tData.bReconnect and not res do
			local err
			
			if wait then ThreadWait(5000) end
			tFuncs.SetStatusText(tLang.Connection.."...")
			tData.Client = socket.tcp()
			
			tData.Client:settimeout(10) -- 10 secs timeout
			res,err = tData.Client:connect(tData.CurrentHub.Address,tonumber(tData.CurrentHub.Port))
			if not res then
				-- These are the most common errors. Translate it to the user's native language
				if err:find("timeout") then -- Connection timeout
					tFuncs.SetStatusText(wx.wxSysErrorMsg(10060))
				elseif err:find("refused") then -- Connection refused by the target machine
					tFuncs.SetStatusText(wx.wxSysErrorMsg(10061))
				elseif err:find("Valid name") then -- crappy address
					tFuncs.SetStatusText(wx.wxSysErrorMsg(10049))
				else
					tFuncs.SetStatusText(err)
				end
			end

			tData.bReconnect = tCfg.Reconnect
			wait = true
		end
		tData.Client:settimeout(nil) -- remove timeout to keep connected
		if tData.TrayIcon then
			tCtrls["taskbar"]:SetIcon(tCtrls["icon_y"],tLang.ConnectionWith..tData.CurrentHub.Name.." ("..tData.CurrentHub.Address..":"..tData.CurrentHub.Port..")")
		end
		tData.bReconnect = nil
	
		local tCommands = tCommands
		local data,e = tData.Client:receive("*p")
			
		while not e do
			local cmd = data:match("^(%$%S+)")
			ThreadLockMutex(mutex)
			if cmd then
				if _log_cmd then Log(">> "..data) end
				if tProtocolCmd[cmd] then
					tHubData.TryUseProtocol = false
					tHubData.AccessDenied = nil
				end
				if tCommands[cmd] then
					tCommands[cmd]((data:gsub("%[endpipe%]","|")))
					wx.wxEndBusyCursor()
				end
			else
				if data:find("^<"..(tHubData.tSettings["chk"][18] == "1" and (tHubData.tSettings["txt"][21] or "PtokaX"):gsub("[%^%$%(%)%%%.%[%]%*%+%-%?]","%%%1") or "Hub%-Security")..">.-%-\r?\nCurrent stats:") then
					tFuncs.ParseStat((data:gsub("%[endpipe%]","|")))
				else
					data = data:gsub("&#124;","|")
					tFuncs.ToChat((data:gsub("%[endpipe%]","|")))
				end
			end
			ThreadUnlockMutex(mutex)

			data,e = tData.Client:receive("*p")
		end
		tData.bReconnect = tCfg.Reconnect
		if tHubData.TryUseProtocol then
			tFuncs.SetStatusText(tLang.MisconfigurationOfServerScript)
			tData.bReconnect = false
		elseif tHubData.AccessDenied then
			tData.bReconnect = false
		end

		tFuncs.SetStatusText(tLang.ConnectionClosed.." ("..tostring(e)..")")
		tData.Client:close()

		tFuncs.CleanUp()
		if not tData.bReconnect then
			tFuncs.CleanUpFinal()
		end
	end
end

function Entered()
	tFuncs.SetStatusText(tLang.Connected)
	for i=0,7 do
		tFuncs.Send("$GET "..i)
	end
	tFuncs.Send("$GET 9")
	tHubData.bEntered = true
	if tData.TrayIcon then
		tCtrls["taskbar"]:SetIcon(tCtrls["icon_g"],tLang.ConnectedWith..tData.CurrentHub.Name)
	end
end

tCommands = {
	-- Standard NMDC extensions
	["$BadPass"] = function(data)
		tFuncs.SetStatusText(tLang.IncorrectPassword)
	end,
	["$GetPass"] = function(data)
		tFuncs.Send("$MyPass "..tData.CurrentHub.Pass)
	end,
	["$Hello"] = function(data)
	    tFuncs.Send("$Version 1,0091|$GetNickList|$MyINFO $ALL "..tData.CurrentHub.Nick..
		" "..tData.CurrentHub.Desc..tData.CurrentHub.Tag.."$ "..
		"$LAN(T3)\1$"..tData.CurrentHub.Email.."$"..tData.CurrentHub.Share.."$")
	end,
	["$HubIsFull"] = function(data)
	    tFuncs.SetStatusText(tLang.HubIsFull)
	end,
	["$HubName"] = function(data)
		local hubname = data:match("^$HubName (.*)$")
		if hubname then
			tCtrls["app"]:SetTitle(_APPNAME.." ".._APPVER.." - "..hubname)
		end
	end,
	["$Lock"] = function(data)
		tFuncs.Send("$Supports NoGetINFO NoHello QuickList")
		local lock = data:match("$Lock%s(%S+)%sPk")
		if lock then tFuncs.Send("$Key "..tFuncs.Lock2Key(lock)) end
	end,
	["$MyINFO"] = function(data)
        if not tHubData.bEntered then
			Entered()
		end
	end,
	["$OpList"] = function(data)
        if not tHubData.bEntered then
			Entered()
		end

		for i in data:gmatch("(%S-)%$%$") do
			tHubData.tOpList[i] = true
		end
	end,
	["$Quit"] = function(data)
		tHubData.tOpList[data:sub(7)] = nil
	end,
	["$Supports"] = function(data)
	    if data:find("QuickList") then
			tFuncs.Send("$MyINFO $ALL "..tData.CurrentHub.Nick.." "..tData.CurrentHub.Desc..
			tData.CurrentHub.Tag.."$ $LAN(T3)\1$"..tData.CurrentHub.Email..
			"$"..tData.CurrentHub.Share.."$")
		else
		    tFuncs.Send("$ValidateNick "..tFuncs.GetCtrl("txt_login_nick"):GetValue())
		end
	end,
	["$To:"] = function(data)
		if data:find("[%--].-Current stats:") then
			tFuncs.ParseStat(data)
		else
			local from,msg = data:match("^$To:%s%S+%sFrom:%s(%S+)%s%$(.+)$")
			tFuncs.ToChat("  "..(from or "?")..": "..(msg or data or ""),"PM")
		end
	end,
	["$UserIP"] = function(data)
	end,
	["ValidateDenide"] = function()
		tFuncs.SetStatusText(tLang.YourNickAlreadyInUse)
	end,
	-- Own protocol extensions start --
	["$SET"] = function(data)
	    local stype,values = data:match("^$SET (%d+) (.+)")
		if not stype then return end
	    stype = tonumber(stype)
		if stype == 1 then -- Booleans
		    local iter = 0
			for id in values:gmatch("([01])%$%$") do
				if tCtrls["chk"]["chk_"..iter] then
					tCtrls["chk"]["chk_"..iter]:Enable(true)
					tCtrls["chk"]["chk_"..iter]:SetValue(id == "1")
					tHubData.tSettings["chk"][iter] = id
					tHubData.tMax.chk = iter
				end
				iter = iter + 1
				if iter > tData.iChkMax then break end
			end
			if not tCtrls["ACCEPT"]:IsEnabled() then tFuncs.EnableControls(true,1) end
            tFuncs.SetStatusText(tLang.BoolsLoaded)
		elseif stype == 2 then -- Numbers
		    local iter = 0
			for id in values:gmatch("(.-)%$%$") do
				if tCtrls["num"]["num_"..iter] then
					tCtrls["num"]["num_"..iter]:Enable(true)
					if tCtrls["num"]["num_"..iter]:GetClassInfo():GetClassName() == "wxSpinCtrl" then
						tCtrls["num"]["num_"..iter]:SetValue(tonumber(id))
					else
						tCtrls["num"]["num_"..iter]:SetSelection(tonumber(id))
					end
					tHubData.tSettings["num"][iter] = id
					tHubData.tMax.num = iter
				end
				iter = iter + 1
				if iter > tData.iNumMax then break end
			end
			if not tCtrls["ACCEPT"]:IsEnabled() then tFuncs.EnableControls(true,1) end
            tFuncs.SetStatusText(tLang.NumbersLoaded)
		elseif stype == 3 then -- Strings
		    if data:find("^$SET 3 lang") then
				tCtrls["txt"]["txt_34"]:Clear()
				tCtrls["txt"]["txt_34"]:Append("Default English")
				for langfile in data:sub(13):gmatch("(.-)%$%$") do
					tCtrls["txt"]["txt_34"]:Append(langfile)
				end
			elseif data:find("^$SET 3 motd") then
				local motd = data:match("^$SET 3 motd (.*)")
				tCtrls["txt"]["txt_motd"]:SetValue(motd)
				tHubData.tSettings["txt"]["motd"] = motd
				return
			else
				local iter = 0
				for id in values:gmatch("(.-)%$%$") do
					if tCtrls["txt"]["txt_"..iter] then
						tCtrls["txt"]["txt_"..iter]:Enable(true)
					    if iter == 34 then
							tCtrls["txt"]["txt_"..iter]:SetStringSelection(id == "nil" and "Default English" or id)
						else
							tCtrls["txt"]["txt_"..iter]:SetValue(id == "nil" and "" or id)
						end
						tHubData.tSettings["txt"][iter] = (id == "nil" and "" or id)
						tHubData.tMax.txt = iter
					end
					iter = iter + 1
					if iter > tData.iTxtMax then break end
				end

				if not tCtrls["ACCEPT"]:IsEnabled() then tFuncs.EnableControls(true,1) end
				for _, v in pairs(tCheckBoxEvtIds) do
					v()
				end
				for _, v in pairs(tChoiceEvtIds) do
					v()
				end
				for _, v in pairs(tSpinCtrlEvtIds) do
					v()
				end

				if not tHubData.bCommandsProcessed and tHubData.tSettings["txt"][29] then
					tFuncs.GetCtrl("ID_CMD_CLIPBOARD"):Enable(true)
					tFuncs.GetCtrl("ID_CMD_REFRESH"):Enable(true)

					tFuncs.Send("<"..tData.CurrentHub.Nick.."> "..tHubData.tSettings["txt"][29]:sub(1,1).."debug off")
					tFuncs.UDPDebugEnable()
					tFuncs.GetStats()

					tHubData.bCommandsProcessed = true
				end
                tFuncs.SetStatusText(tLang.StringsLoaded)
			end
		elseif stype == 4 then -- Bans
		    local subtype,values = data:match("^$SET 4 (%d) (.+)")
			subtype = tonumber(subtype)
			if subtype == 1 then -- Permanent bans
				for sIP,sNick,bFullIpBan,sBy,sReason in data:sub(10):gmatch("([^%$]*)%$([^%$]*)%$([01])%$([^%$]*)%$([^%$]*)%$%$") do
					sReason = sReason:gsub(string.char(10),""):gsub(string.char(13),"")
					table.insert(tHubData.tBans,{sIP,sNick,sBy,sReason,"",bFullIpBan})
					local item = tCtrls["lst_bans"]:InsertItem(tCtrls["lst_bans"]:GetItemCount(),
					tFuncs.IsIP(sIP) and (bFullIpBan == "1" and sIP.." ("..tLang.FullBan..")" or sIP.." ("..tLang.Banned..")") or "")
					tCtrls["lst_bans"]:SetItem(item,1,(#sNick == 0 and "" or sNick.." ("..tLang.Banned..")"))
					tCtrls["lst_bans"]:SetItem(item,2,sBy)
					tCtrls["lst_bans"]:SetItem(item,3,sReason)
					tCtrls["lst_bans"]:SetItem(item,4,"")
					tCtrls["lst_bans"]:SetItemData(item,tCtrls["lst_bans"]:GetItemCount()-1)
				end
				if not tCtrls["REFRESH_BANS"]:IsEnabled() then tFuncs.EnableControls(true,5) end
                tFuncs.SetStatusText(tLang.PermBansLoaded)
			elseif subtype == 2 then -- Temporary bans
				for sIP,sNick,bFullIpBan,sBy,sReason,iExpireTime in data:sub(10):gmatch("([^%$]*)%$([^%$]*)%$([01])%$([^%$]*)%$([^%$]*)%$(%d+)%$%$") do
					table.insert(tHubData.tBans,{sIP,sNick,sBy,sReason,iExpireTime,bFullIpBan})
					local item = tCtrls["lst_bans"]:InsertItem(tCtrls["lst_bans"]:GetItemCount(),
					tFuncs.IsIP(sIP) and (bFullIpBan == "1" and sIP.." ("..tLang.FullBan..")" or sIP.." ("..tLang.Banned..")") or "")
					tCtrls["lst_bans"]:SetItem(item,1,(#sNick == 0 and "" or sNick.." ("..tLang.Banned..")"))
					tCtrls["lst_bans"]:SetItem(item,2,sBy)
					tCtrls["lst_bans"]:SetItem(item,3,sReason)
					tCtrls["lst_bans"]:SetItem(item,4,os.date("%Y. %b %d. %H:%M:%S",tonumber(iExpireTime)))
					tCtrls["lst_bans"]:SetItemData(item,tCtrls["lst_bans"]:GetItemCount()-1)
				end
				if not tCtrls["REFRESH_BANS"]:IsEnabled() then tFuncs.EnableControls(true,5) end
                tFuncs.SetStatusText(tLang.TempBansLoaded)
			elseif subtype == 3 then -- Permanent rangebans
				for sIPFrom,sIPTo,bFullIpBan,sBy,sReason in data:sub(10):gmatch("([^%$]*)%$([^%$]*)%$([01])%$([^%$]*)%$([^%$]*)%$%$") do
					table.insert(tHubData.tRangeBans,{sIPFrom.." - "..sIPTo,sBy,sReason,"",bFullIpBan})
					local item = tCtrls["lst_rangebans"]:InsertItem(tCtrls["lst_rangebans"]:GetItemCount(),
					(bFullIpBan == "1" and sIPFrom.." - "..sIPTo.." ("..tLang.FullBan..")" or sIPFrom.." - "..sIPTo))
					tCtrls["lst_rangebans"]:SetItem(item,1,sBy)
					tCtrls["lst_rangebans"]:SetItem(item,2,sReason)
					tCtrls["lst_rangebans"]:SetItem(item,3,"")
					tCtrls["lst_rangebans"]:SetItemData(item,tCtrls["lst_rangebans"]:GetItemCount()-1)
				end
				if not tCtrls["REFRESH_RANGEBANS"]:IsEnabled() then tFuncs.EnableControls(true,6) end
                tFuncs.SetStatusText(tLang.PermRangeBansLoaded)
			elseif subtype == 4 then -- Temporary rangebans
				for sIPFrom,sIPTo,bFullIpBan,sBy,sReason,iExpireTime in data:sub(10):gmatch("([^%$]*)%$([^%$]*)%$([01])%$([^%$]*)%$([^%$]*)%$(%d+)%$%$") do
					table.insert(tHubData.tRangeBans,{sIPFrom.." - "..sIPTo,sBy,sReason,iExpireTime,bFullIpBan})
					local item = tCtrls["lst_rangebans"]:InsertItem(tCtrls["lst_rangebans"]:GetItemCount(),
					(bFullIpBan == "1" and sIPFrom.." - "..sIPTo.." ("..tLang.FullBan..")" or sIPFrom.." - "..sIPTo))
					tCtrls["lst_rangebans"]:SetItem(item,1,sBy)
					tCtrls["lst_rangebans"]:SetItem(item,2,sReason)
					tCtrls["lst_rangebans"]:SetItem(item,3,os.date("%Y. %b %d. %H:%M:%S",tonumber(iExpireTime)))
					tCtrls["lst_rangebans"]:SetItemData(item,tCtrls["lst_rangebans"]:GetItemCount()-1)
				end
				if not tCtrls["REFRESH_RANGEBANS"]:IsEnabled() then tFuncs.EnableControls(true,6) end
                tFuncs.SetStatusText(tLang.TempRangeBansLoaded)
			end
			-- Fetch the controls
		elseif stype == 5 then
			tCtrls["mem_table"] = {}
			tCtrls["lst_scripts"]:Clear()
			tCtrls["tScripts"] = {}
			for sName,bEnabled,iMemUsage in data:sub(8):gmatch("([^%$]*)%$([01])%$(%d+)%$%$") do
				tCtrls["lst_scripts"]:Insert(sName,tCtrls["lst_scripts"]:GetCount())
				tCtrls["tScripts"][sName] = true
				if bEnabled == "1" then
					table.insert(tCtrls["mem_table"],{sName,iMemUsage})
					tCtrls["lst_scripts"]:Check(tCtrls["lst_scripts"]:GetCount() - 1,true)
				end
			end
			tCtrls["lst_scripts"]:Refresh()
			tFuncs.EnableControls(true,2)
            tFuncs.SetStatusText(tLang.ScriptsLoaded)
		elseif stype == 6 then
			for iProfileNumber,sProfileName,sProfilePermissions in data:sub(8):gmatch("(%d+)%$([^%$]*)%$([01]-)%$%$") do
				tHubData.tProfiles[tonumber(iProfileNumber)] = {sProfileName,sProfilePermissions}
				tCtrls["lst_profiles"]:Append(sProfileName)
				if tonumber(iProfileNumber) == 0 then
					tFuncs.LoadPermissions(iProfileNumber)
				end
				tCtrls["lst_profiles"]:SetSelection(0)
			end
			tFuncs.EnableControls(true, 4)
			tFuncs.GetCtrl("wxID_PROF_UP"):Enable(false)
            tFuncs.SetStatusText(tLang.ProfilesLoaded)
		elseif stype == 7 then
			for sNick,sPassword,iProfile in data:sub(8):gmatch("([^%$]+)%$([^%$]*)%$(%d+)%$%$") do
				if tHubData.tProfiles[tonumber(iProfile)] then
					table.insert(tHubData.tRegs,{sNick,sPassword,tHubData.tProfiles[tonumber(iProfile)][1]})
					local item = tCtrls["lst_regs"]:InsertItem(tCtrls["lst_regs"]:GetItemCount(),sNick)
					tCtrls["lst_regs"]:SetItem(item,1,sPassword)
					tCtrls["lst_regs"]:SetItem(item,2,tHubData.tProfiles[tonumber(iProfile)][1])
					tCtrls["lst_regs"]:SetItemData(item,tCtrls["lst_regs"]:GetItemCount()-1)
				end
			end
			tFuncs.EnableControls(true,3)
            tFuncs.SetStatusText(tLang.RegsLoaded)
		elseif stype == 8 then
			tHubData.tTexts = {}
			tCtrls["lst_texts"]:Clear()
			for textfile in data:sub(8):gmatch("(.-)%$%$") do
				tHubData.tTexts[textfile] = true
				tCtrls["lst_texts"]:Append(textfile)
			end
			tFuncs.EnableControls(true,7)
            tFuncs.SetStatusText(tLang.TextFilesLoaded)
		elseif stype == 9 then
			tHubData.sScriptName = data:sub(8)
		elseif stype == 10 then
			tFuncs.EnableControls(true,8)
		end
	end,
	["$FILE"] = function(data)
		local file,content = data:match("^$FILE (.-):(.*)$")
		local p = ""
		if file:lower():sub(-4) == ".lua" then
			p = _save_path.."scripts/"
		else
			p = _save_path.."texts/"
		end
		content = content:gsub("%[endpipe%]","|")
		local f = io.open(p..file,"wb")
		f:write(content)
		f:close()
		tHubData.tFiles[file] = 1
	end,
	["$ERR"] = function(data) -- Errors
		local level = data:match("^$ERR (%d+)")
		level = tonumber(level)
		local tHandlers = {
		[1] = function()
		    -- Requested file couldn't find
			tFuncs.SetStatusText(tLang.FileNotFound.."\n")
			tFuncs.ReloadScripts()
			tFuncs.ReloadTexts()
		end,
		[2] = function()
		    -- MoveDown() failed
			tCtrls["txt_scripterror"]:AppendText("["..os.date("%H:%M:%S").."] "..tLang.UnableToMoveScriptDown.."\n")
			tFuncs.ReloadScripts()
		end,
		[3] = function()
		    -- MoveUp() failed
			tCtrls["txt_scripterror"]:AppendText("["..os.date("%H:%M:%S").."] "..tLang.UnableToMoveScriptUp.."\n")
			tFuncs.ReloadScripts()
		end,
		[4] = function()
		    -- StartScript() failed
		    local what = data:match("^$ERR 4 (.+)$")
		    local iPos,bChecked = tCtrls["lst_scripts"]:FindString(what),tCtrls["lst_scripts"]:IsChecked(tCtrls["lst_scripts"]:FindString(what))
			tCtrls["lst_scripts"]:Check(iPos,false)
			tCtrls["txt_scripterror"]:AppendText("["..os.date("%H:%M:%S").."] "..tLang.UnableToStartScript..what.."\n")
		end,
		[5] = function()
		    --> RestartScript(s)() failed
			tCtrls["txt_scripterror"]:AppendText("["..os.date("%H:%M:%S").."] "..tLang.UnableToRestartScript.."\n")
			tFuncs.ReloadScripts()
		end,
		[6] = function()
		    --> MoveUp() failed
		    wx.wxMessageBox(tLang.UnableToMoveProfileUp,tLang.Error,wx.wxICON_WARNING)
		    tCtrls["lst_profiles"]:Clear()
		    tFuncs.Send("$GET 5")
		    if tCtrls["lst_profiles"]:GetCount() == 1 then
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(false)
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(false)
			elseif tCtrls["lst_profiles"]:IsSelected(0) then
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(false)
			elseif tCtrls["lst_profiles"]:IsSelected(tCtrls["lst_profiles"]:GetCount()-1) then
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(false)
			else
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(true)
			end
		end,
		[7] = function()
		    --> MoveDown() failed
		    wx.wxMessageBox(tLang.UnableToMoveProfileDown,tLang.Error,wx.wxICON_WARNING)
		    tCtrls["lst_profiles"]:Clear()
		    tFuncs.Send("$GET 5")
		    if tCtrls["lst_profiles"]:GetCount() == 1 then
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(false)
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(false)
			elseif tCtrls["lst_profiles"]:IsSelected(0) then
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(false)
			elseif tCtrls["lst_profiles"]:IsSelected(tCtrls["lst_profiles"]:GetCount()-1) then
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(false)
			else
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(true)
			end
		end,
		[8] = function()
		    --> AddProfile failed
		    wx.wxMessageBox(tLang.UnableToAddProfile,tLang.Error,wx.wxICON_ERROR) --  Why? Don't ask... Maybe already exists?
		end,
		[9] = function()
		    --> RemoveProfile() failed
		    -- We checked this when the button pressed. But who knows? :-)
		    wx.wxMessageBox(tLang.ProfileDelFailed)
		end,
		[10] = function()
		    --> (Range)Unban() failed
		    local sublevel,position = data:match("^$ERR 10 ([01])$")
		    if sublevel then
				if sublevel == "0" then
					wx.wxMessageBox(tLang.UnableToUnban,tLang.Error,wx.wxICON_WARNING)
					tCtrls["lst_bans"]:DeleteAllItems()
					tFuncs.Send("$GET 3 0")
				elseif sublevel == "1" then
					wx.wxMessageBox(tLang.UnableToUnbanRange,tLang.Error,wx.wxICON_WARNING)
					tCtrls["lst_rangebans"]:DeleteAllItems()
					tFuncs.Send("$GET 3 1")
				end
			end
		end,
		[11] = function()
		    local ID = data:match("^$ERR 11 (%d+)$")
		    if ID then
			  ID = tonumber(ID)
			  tCtrls["lst_permissions"]:Check(tCtrls["lst_permissions"]:IsChecked(ID))
			end
		end,
		[12] = function()
		    --> DelReg() failed
		    local nick = data:match("^$ERR 12 (%S+)")
			wx.wxMessageBox(tLang.UnableToDeleteReg..nick,tLang.Error,wx.wxICON_WARNING)
		end,
		[13] = function()
		    --> AddReg() failed
		    local nick = data:match("^$ERR 13 (%S+)$")
			wx.wxMessageBox(nick..tLang.AlreadyRegistered,tLang.Error,wx.wxICON_WARNING)
		end,
		[14] = function()
		    --> ChangeReg() failed
		    local nick = data:match("^$ERR 14 (%S+)$")
			wx.wxMessageBox(tLang.UnableToChangeReg..nick,tLang.Error,wx.wxICON_WARNING)
		end,
		[15] = function()
		    --> IP-To-Country reload failed
			wx.wxMessageBox(tLang.UnableToReloadIP2C,tLang.Error,wx.wxICON_WARNING)
		end,
		[16] = function()
			if tHubData.IP2C then
				if tHubData.IP2C.File and io.type(tHubData.IP2C.File) == "file" then
					tHubData.IP2C.File:close()
				end
				tHubData.IP2C = nil
				wx.wxMessageBox(tLang.UploadIP2CError,tLang.Error,wx.wxICON_WARNING)
				tCtrls["IP2C_UPLOADV4"]:Enable(true)
				tCtrls["IP2C_UPLOADV6"]:Enable(true)
				tCtrls["IP2C_RELOAD"]:Enable(true)
			end
		end,
		[17] = function()
			tFuncs.SetStatusText(tLang.RegListTooLong)
		end,
		[99] = function()
			local err_cat = {tLang.Settings,tLang.Scripting,tLang.RegUsers,tLang.Profiles,tLang.Bans,tLang.RangeBans,tLang.TextFiles,"IP-To-Country"}
			local cat = data:match("^$ERR 99 (%d+)")
			if cat and err_cat[tonumber(cat)] then
				tFuncs.SetStatusText(err_cat[tonumber(cat)]..": "..tLang.AccessDenied)
			else
				tFuncs.SetStatusText(tLang.AccessDenied)
				tHubData.AccessDenied = true
			end
		end
		}
		if tHandlers[level] then tHandlers[level]() end
	end,
	["$CONF"] = function(data) -- Confirmations
		local level,what = data:match("^$CONF (%d+) (.*)$")
		if not level then return end
		level = tonumber(level)
		local tConfirmations = {
		[2] = function()
		    --> MoveDown() success
			local iPos,bChecked = tCtrls["lst_scripts"]:GetSelection(),tCtrls["lst_scripts"]:IsChecked(tCtrls["lst_scripts"]:GetSelection())
			tCtrls["lst_scripts"]:Delete(iPos)
			tCtrls["lst_scripts"]:Insert(what,iPos + 1)
			tCtrls["lst_scripts"]:Check(iPos + 1,bChecked)
			tCtrls["lst_scripts"]:Select(iPos + 1)
			if tCtrls["lst_scripts"]:IsSelected(0) then
				tFuncs.GetCtrl("wxID_LUA_DOWN"):Enable(true)
				tFuncs.GetCtrl("wxID_LUA_UP"):Enable(false)
			elseif tCtrls["lst_scripts"]:IsSelected(tCtrls["lst_scripts"]:GetCount()-1) then
				tFuncs.GetCtrl("wxID_LUA_UP"):Enable(true)
				tFuncs.GetCtrl("wxID_LUA_DOWN"):Enable(false)
			end
		end,
		[3] = function()
		    --> MoveUp() success
			local iPos,bChecked = tCtrls["lst_scripts"]:GetSelection(),tCtrls["lst_scripts"]:IsChecked(tCtrls["lst_scripts"]:GetSelection())
			tCtrls["lst_scripts"]:Delete(iPos)
			tCtrls["lst_scripts"]:Insert(what,iPos - 1)
			tCtrls["lst_scripts"]:Check(iPos - 1,bChecked)
			tCtrls["lst_scripts"]:Select(iPos - 1)
			if tCtrls["lst_scripts"]:IsSelected(0) then
				tFuncs.GetCtrl("wxID_LUA_DOWN"):Enable(true)
				tFuncs.GetCtrl("wxID_LUA_UP"):Enable(false)
			elseif tCtrls["lst_scripts"]:IsSelected(tCtrls["lst_scripts"]:GetCount()-1) then
				tFuncs.GetCtrl("wxID_LUA_UP"):Enable(true)
				tFuncs.GetCtrl("wxID_LUA_DOWN"):Enable(false)
			end
		end,
		[4] = function()
		    --> StartScript()/StartScript() success
			local iPos,bChecked = tCtrls["lst_scripts"]:FindString(what),tCtrls["lst_scripts"]:IsChecked(tCtrls["lst_scripts"]:FindString(what))
			if bChecked then
				tCtrls["lst_scripts"]:Check(iPos,true)
				tCtrls["txt_scripterror"]:AppendText("["..os.date("%H:%M:%S").."] "..tLang.ScriptStarted..".\n")
			else
			    tCtrls["lst_scripts"]:Check(iPos,false)
				tCtrls["txt_scripterror"]:AppendText("["..os.date("%H:%M:%S").."] "..tLang.ScriptStopped..".\n")
			end
		end,
		[5] = function()
		    --> RestartScript(s)() success
			tCtrls["txt_scripterror"]:AppendText("["..os.date("%H:%M:%S").."] "..what.." "..tLang.RestartedLwr.."\n")
		end,
		[6] = function()
		    --> MoveDown() success
		    local iPos,sProfile = tonumber(what),tCtrls["lst_profiles"]:GetStringSelection()
		    tCtrls["lst_profiles"]:Delete(iPos)
		    tCtrls["lst_profiles"]:Insert(sProfile,iPos+1)
		    tCtrls["lst_profiles"]:SetSelection(iPos+1)
			local temp = tHubData.tProfiles[iPos+1]
			tHubData.tProfiles[iPos+1] = tHubData.tProfiles[iPos]
			tHubData.tProfiles[iPos] = temp
		    if iPos-1 == 0 then
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(false)
			elseif iPos == tCtrls["lst_profiles"]:GetCount() then
			    tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(false)
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(true)
			else
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(true)
			end
		end,
		[7] = function()
		    --> MoveUp() success
		    local iPos,sProfile = tonumber(what),tCtrls["lst_profiles"]:GetStringSelection()
		    tCtrls["lst_profiles"]:Delete(iPos)
		    tCtrls["lst_profiles"]:Insert(sProfile,iPos-1)
		    tCtrls["lst_profiles"]:SetSelection(iPos-1)
			local temp = tHubData.tProfiles[iPos-1]
			tHubData.tProfiles[iPos-1] = tHubData.tProfiles[iPos]
			tHubData.tProfiles[iPos] = temp
		    if iPos-1 == 0 then
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(false)
			elseif iPos == tCtrls["lst_profiles"]:GetCount() then
			    tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(false)
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(true)
			else
				tFuncs.GetCtrl("wxID_PROF_DOWN"):Enable(true)
				tFuncs.GetCtrl("wxID_PROF_UP"):Enable(true)
			end
		end,
		[8] = function()
		    --> AddProfile() success
		    tCtrls["lst_profiles"]:Insert(what,tCtrls["lst_profiles"]:GetCount())
		    table.insert(tHubData.tProfiles,{what,string.rep("0",47)})
		end,
		[9] = function()
			--> RemoveProfile() success
		    local item = tCtrls["lst_profiles"]:FindString(what)
		    tCtrls["lst_profiles"]:SetSelection(item-1)
		    tFuncs.LoadPermissions(item-1)
		    tCtrls["lst_profiles"]:Delete(item)
		    table.remove(tHubData.tProfiles,item-1)
		end,
		[10] = function()
		    --> (Range)Unban() success
		    local subtype,item = data:match("^$CONF 10 ([0-3]) (%d*)")
			if not subtype then return end
			item = tonumber(item)
		    if subtype then
			    if subtype == "0" then
					local i = tCtrls["lst_bans"]:GetItemData(item)+1
					tCtrls["lst_bans"]:DeleteItem(item)
					table.remove(tHubData.tBans,i)
					tFuncs.ReIndex("lst_bans",i)
				elseif subtype == "1" then
				    local i = tCtrls["lst_rangebans"]:GetItemData(item)+1
					tCtrls["lst_rangebans"]:DeleteItem(item)
					table.remove(tHubData.tRangeBans,i)
					tFuncs.ReIndex("lst_rangebans",i)
				-- All wxListCtrl item removing must be done backwards
				-- as in associated table so as in the listview.
			    elseif subtype == "2" then
					for i=#tHubData.tBans,1,-1 do
						if not tHubData.tBans[i][5]:find("%b<>") then
							for j=tCtrls["lst_bans"]:GetItemCount()-1,0,-1 do
								if tCtrls["lst_bans"]:GetItemData(j) == i-1 then
									tCtrls["lst_bans"]:DeleteItem(j)
									table.remove(tHubData.tBans,i)
									tFuncs.ReIndex("lst_bans",i)
								end
							end
						end
					end
				elseif subtype == "3" then
					for i=#tHubData.tBans,1,-1 do
						if tHubData.tBans[i][5]:find("%b<>") then
							for j=tCtrls["lst_bans"]:GetItemCount()-1,0,-1 do
								if tCtrls["lst_bans"]:GetItemData(j) == i-1 then
									tCtrls["lst_bans"]:DeleteItem(j)
									table.remove(tHubData.tBans,i)
									tFuncs.ReIndex("lst_bans",i)
								end
							end
						end
					end
				elseif subtype == "4" then
					for i=#tHubData.tRangeBans,1,-1 do
						if not tHubData.tRangeBans[4]:find("%b<>") then
							for j=tCtrls["lst_rangebans"]:GetItemCount()-1,0,-1 do
								if tCtrls["lst_rangebans"]:GetItemData(j) == i-1 then
									tCtrls["lst_rangebans"]:DeleteItem(j)
									table.remove(tHubData.tRangeBans,i)
									tFuncs.ReIndex("lst_rangebans",i)
								end
							end
						end
					end
				elseif subtype == "5" then
					for i=#tHubData.tRangeBans,1,-1 do
						if tHubData.tRangeBans[4]:find("%b<>") then
							for j=tCtrls["lst_rangebans"]:GetItemCount()-1,0,-1 do
								if tCtrls["lst_rangebans"]:GetItemData(j) == i-1 then
									tCtrls["lst_rangebans"]:DeleteItem(j)
									table.remove(tHubData.tRangeBans,i)
									tFuncs.ReIndex("lst_rangebans",i)
								end
							end
						end
					end
				end
			end
		end,
		[11] = function()
		    --> SetProfilePermission() success
			local ID,value = data:match("^$CONF 11 (%d+) ([01])$")
			ID = tonumber(ID)
			if ID == 56 then
				for i=0,55 do
					tCtrls["lst_permissions"]:Check(i,(value == "1"))
				end
				local profilestring = tHubData.tProfiles[tCtrls["lst_profiles"]:GetSelection()][2]
				tHubData.tProfiles[tCtrls["lst_profiles"]:GetSelection()][2] = string.rep("1",55)
				return
			end
			tCtrls["lst_permissions"]:Check(ID,(value == "1"))
			local profilestring = tHubData.tProfiles[tCtrls["lst_profiles"]:GetSelection()][2]
			tHubData.tProfiles[tCtrls["lst_profiles"]:GetSelection()][2] = profilestring:sub(1,ID)..value..profilestring:sub(ID+2)
		end,
		[12] = function()
		    --> DelReg() success
		    local nick = data:match("^$CONF 12 ([^ ]+)")
			if nick then
			    local item = tCtrls["lst_regs"]:FindItem(-1,nick,false)
			    tCtrls["lst_regs"]:DeleteItem(item)
				for i,v in ipairs(tHubData.tRegs) do
					if v[1] == nick then
						table.remove(tHubData.tRegs,i)
						tFuncs.ReIndex("lst_regs",i)
						break
					end
				end
			end
		end,
		[13] = function()
		    --> AddReg() success
		    local nick,pass,profile = data:match("^$CONF 13 ([^ ]+) ([^ ]*) (%d+)$")
			if nick then
			    local profilename = tHubData.tProfiles[tonumber(profile)][1]
				local count = tCtrls["lst_regs"]:GetItemCount()
			    local pos = tCtrls["lst_regs"]:InsertItem(count,nick,0)
				table.insert(tHubData.tRegs,{nick,pass,profilename})
			    tCtrls["lst_regs"]:SetItem(pos,1,pass)
			    tCtrls["lst_regs"]:SetItem(pos,2,profilename)
			    tCtrls["lst_regs"]:SetItemImage(pos,-1)
			    tCtrls["lst_regs"]:SetItemData(pos,count)
			end
		end,
		[14] = function()
		    --> ChangeReg() success
		    local nick,pass,profile = data:match("^$CONF 14 ([^ ]+) ([^ ]*) (%d+)$")
			if nick then
			    local profilename = tHubData.tProfiles[tonumber(profile)][1]
			    local pos = tCtrls["lst_regs"]:FindItem(-1,nick,false)
			    tCtrls["lst_regs"]:SetItem(pos,1,pass)
			    tCtrls["lst_regs"]:SetItem(pos,2,profilename)
				for i,v in ipairs(tHubData.tRegs) do
					if v[1] == nick then
						tHubData.tRegs[i] = {nick,pass,profilename}
						break
					end
				end
			end
		end,
		[15] = function()
			--> IP2Country.Reload() success
			tFuncs.SetStatusText(tLang.IP2CReloaded)
		end,
		[16] = function()
			if tHubData.IP2C then
				what = tonumber(what)
				if not what then return end
				if what == 0 then
					tFuncs.SetStatusText(tLang.IP2CUploaded:format(tHubData.IP2C.FileType == 1 and "IPv4" or "IPv6"))
					tHubData.IP2C = nil
					tCtrls["IP2C_UPLOADV4"]:Enable(true)
					tCtrls["IP2C_UPLOADV6"]:Enable(true)
					tCtrls["IP2C_RELOAD"]:Enable(true)
				else
					tCtrls["statusbar"]:SetStatusText(tLang.IP2CUploadedPercents:format(math.floor(what*tData.iIP2CFilePart/tHubData.IP2C.FileSize*100)),1)
					tFuncs.UploadIP2CPart(what+1)
				end
			end
		end,
		}
		if tConfirmations[level] then
			tConfirmations[level]()
		end
	end,
}
